
void DBG32(uint32_t num){
	uint8_t tmp;
	int i;
	for(i=0;i<4;i++){
		tmp = num>>(8*i);
		DBGNUM(tmp);
		DBG(".");
	}
	DBG("\n");
}
		
/* fills the buffer with the user message
return values:
-1 error, if not enough data in user request
0 success, no gpio response needed
1 success, gpio response requested
*/
int getBufferGPIO(int8_t *buffer)
{
	int i, count=0;
	
	//getting next 2 bytes of the protocol - port, operation
	buffer[0] = vcomGetchar(); //port
	if (buffer[0] == -1)//error
		return -1;
	DBG("port: ");
	DBGNUM(buffer[0]);
	DBG("\n");

	buffer[1] = vcomGetchar(); //operation
	if (buffer[1] == -1)//error
		return -1;
	DBG("buffer: ");
	DBGNUM(buffer[1]);
	DBG("\n");

	if (buffer[1] & OPERATION_DIR) //getting direction bytes		
	{
		for (i=0; i< 8; i++, count++)
		{
			buffer[count+2] = vcomGetchar();
			if (buffer[count+2] == -1)//error
				return -1;
			DBG("buffer: ");
			DBGNUM(count+2);
			DBG(" ");
			DBGNUM(buffer[count+2]);
			DBG("\n");
		}
	}
	if (buffer[1] & OPERATION_SC) //getting set clear bytes		
	{
		for (i=0; i< 8; i++, count++)
		{
			buffer[count+2] = vcomGetchar();
			if (buffer[count+2] == -1)//error
				return -1;
			DBG("buffer: ");
			DBGNUM(count+2);
			DBG(" ");
			DBGNUM(buffer[count+2]);
			DBG("\n");
		}
	}
	DBG("return fro getBufferGPIO\n");
	return 0;
}

/*
handles gpio request
return values:
-1 error, conflict in user data
0 success, no gpio response
1 success, return_val contains gpio response
*/
int handleGPIO(int8_t *ins_buffer, uint32_t *return_val)
{
	uint32_t check1, check2, tmp;
	int count=0;
	int i;

	if (ins_buffer[1] & OPERATION_DIR) //check for conflicts in DIR bytes
	{
		memcpy(&check1, ins_buffer+2, 4);
		memcpy(&check2, ins_buffer+6, 4);
		if ((check1 &  check2) != 0)
			return -1;
		count = 8;
	}
	if (ins_buffer[1] & OPERATION_SC) //check for conflict in SET CLEAR bytes
	{
		memcpy(&check1, ins_buffer+2+count, 4);
		memcpy(&check2, ins_buffer+6+count, 4);
		if ((check1 &  check2) != 0)
			return -1;
	}	
	count = 0;	
	if (ins_buffer[1] & OPERATION_DIR) //handle direction operation
	{
		DBG("in dir\n");		
		memcpy(&check1, ins_buffer+2, 4);
		memcpy(&check2, ins_buffer+6, 4);
		count = 8;		
		switch(ins_buffer[0])
		{
			case PORT0:
				tmp = 0xffffffff & IODIR0;
				DBG("tmp: ");
				DBG32(tmp);
				DBG("check1: ");
				DBG32(check1);
				DBG("check2: ");
				DBG32(check2);
				tmp &= ~check1;
				tmp |= check2;
				DBG("tmp: ");
				DBG32(tmp);
				IODIR0 = 0xffffffff & tmp;
				break;
			case PORT1:
				tmp = 0xffffffff & IODIR1;
				tmp &= ~check1;
				tmp |= check2;
				IODIR1 = 0xffffffff & tmp;
				break;
			default:
				return -1;
		}
	}
	if (ins_buffer[1] & OPERATION_SC) //handle set/clear operation
	{
		DBG("in sc\n");		
		memcpy(&check1, ins_buffer+2+count, 4);
		memcpy(&check2, ins_buffer+6+count, 4);
		switch(ins_buffer[0])
		{
			case PORT0:
				tmp = 0xffffffff & IOPIN0;
				DBG("tmp: ");
				DBG32(tmp);
				DBG("check1: ");
				DBG32(check1);
				DBG("check2: ");
				DBG32(check2);
				tmp |= check2;
				tmp &= ~check1;
				DBG("tmp: ");
				DBG32(tmp);
				IOPIN0 = 0xffffffff & tmp;
				break;
			case PORT1:
				tmp = 0xffffffff & IOPIN1;
				tmp |= check2;
				tmp &= ~check1;
				IOPIN1 = 0xffffffff & tmp;
				break;
			default:
				return -1;
		}
	}
	if (ins_buffer[1] & OPERATION_VAL) //handle getValue operation
	{
		*return_val = 0xffffffff & IOPIN0;
		return 1;
	}
	return 0;	
}
